#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <commdlg.h>
#include <tapi.h>
#include <io.h>
#include <stdlib.h>
#include <malloc.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include "tam.h"


/* quick and dirty RIFF chunk */
BYTE ab_riffchunk[] = {'R','I','F','F',  0,0,0,0, 'W','A','V','E'};
/* quick and dirty format chunk tag */
BYTE ab_formatchunktag[] = {'f','m','t',' ',  0,0,0,0};
/* quick and dirty data chunk header */
BYTE ab_datachunktag[] = {'d','a','t','a',  0,0,0,0};

/* message headers */
WAVEHDR inMessageHeader = {NULL, 0, 0, NULL, 0 /* no flags */, 0 /* no loops */, NULL, 0};

/* handle to main window */
extern HWND h_wndmain;

/* format of in-message */
PCMWAVEFORMAT messageFormat = {WAVE_FORMAT_PCM, 1, 22050, 22050, 1, 8};

/****************************************************************************
    FUNCTION: recordMessage
    PURPOSE:  begin recording voice message
****************************************************************************/
/* record the voice message  - 60s max */
HWAVEIN recordMessage (DWORD dw_devid)
{
HWAVEIN h_wavein; 
HDRVR hdrv;
DWORD dw_60sbufsz = 60L * (LONG)messageFormat.wf.nSamplesPerSec * 
							(LONG)messageFormat.wBitsPerSample/8L;


	/* open recorder. */
	if (waveInOpen(	&h_wavein, (WORD) dw_devid,	(LPWAVEFORMAT)&messageFormat,
					(DWORD)h_wndmain, NULL, CALLBACK_WINDOW)) {
        MessageBox (NULL, "Error opening wave record device.", "", MB_OK);
        return 0;
	}

	/* allocate 60s message buffer */
	if ((inMessageHeader.lpData = (HPSTR) halloc (dw_60sbufsz, 1)) == NULL) {
        MessageBox (NULL, "Not enough memory.", "", MB_OK);
        return 0;
	}
	inMessageHeader.dwBufferLength = dw_60sbufsz;
	if (waveInPrepareHeader(h_wavein, &inMessageHeader, sizeof(WAVEHDR))) {
        MessageBox (NULL, "Error preparing message header.", "", MB_OK);
        return 0;
	}

	/* pass down a 60s buffer to record into */
	if (waveInAddBuffer (h_wavein, &inMessageHeader, sizeof(WAVEHDR))) {
        MessageBox (NULL, "Error adding buffer.", "", MB_OK);
        return 0;
	}
	/* start recording */
	waveInStart (h_wavein);

	return h_wavein;
	
} /* end function (record message) */
/****************************************************************************
	FUNCTION: saveMessage
	PURPOSE:  save a recorded voice message
****************************************************************************/
/* save the message after the caller hangs up, or after 60s */
int saveMessage ()
{
HANDLE h_file;
static int MsgNum=1;
char sz_fname[_MAX_PATH+1];

	wsprintf(sz_fname,"call-%d.wav",MsgNum++);
    h_file = open (sz_fname, O_CREAT | O_BINARY | O_WRONLY);
	if (h_file==-1) {
        MessageBox (NULL, "Error opening file.", "", MB_OK);
        return -1;
    }

	/* write out the RIFF chunk */
	*((DWORD *)&ab_riffchunk[4]) = 4 + sizeof (ab_formatchunktag) + sizeof (messageFormat)
									+ sizeof (ab_datachunktag) + inMessageHeader.dwBytesRecorded;
    write(h_file, ab_riffchunk, sizeof(ab_riffchunk));

	/* write out the format chunk tag */
	*((DWORD *)&ab_formatchunktag[4]) = sizeof (messageFormat);
    write(h_file, ab_formatchunktag, sizeof(ab_formatchunktag));

	/* write out the canned format header */
	write (h_file, &messageFormat, sizeof(messageFormat));

	/* write out the data chunk tag */
	*((DWORD *)&ab_datachunktag[4]) = inMessageHeader.dwBytesRecorded;
    write(h_file, ab_datachunktag, sizeof(ab_datachunktag));

	/* write out the data chunk */
	_hwrite (h_file, inMessageHeader.lpData, inMessageHeader.dwBytesRecorded);

	/* close message file */
	close (h_file);
	return 0;

} /* end function (save message) */

