#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <tapi.h>
#include "tam.h"

/****************************************************************************
    FUNCTION: 
    PURPOSE:  
****************************************************************************/
/* begin playing out the message */
HWAVEOUT playSound (DWORD dw_devid, char *szWaveFile, HWND hWnd)
{
HWAVEOUT		hWave;
HDRVR 			hdrv;
HMMIO			hmmio;
MMCKINFO		mmckinfoParent;
MMCKINFO		mmckinfoSubchunk;
DWORD			dwFmtSize;
DWORD			dwDataSize;
HPSTR			lpWaveData;
WAVEHDR FAR*	lpWaveHdr;
PCMWAVEFORMAT FAR *lpWaveFormat;
	
	/* Open wave file */
	hmmio = mmioOpen(szWaveFile, NULL, MMIO_READ | MMIO_ALLOCBUF);
	if(!hmmio)
	{
		return 0;
	}

	/* Locate a 'RIFF' chunk with a 'WAVE' form type */
	mmckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E');
	if (mmioDescend(hmmio, (LPMMCKINFO) &mmckinfoParent, NULL, MMIO_FINDRIFF))
	{
		mmioClose(hmmio, 0);
		return 0;
	}
	
	/* Find the format chunk */
	mmckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	if (mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK))
	{
		mmioClose(hmmio, 0);
		return 0;
	}

	/* Get the size of the format chunk, allocate memory for it */
	dwFmtSize = mmckinfoSubchunk.cksize;
	lpWaveFormat = (PCMWAVEFORMAT FAR *) GlobalAllocPtr(GMEM_MOVEABLE, dwFmtSize);
	if (!lpWaveFormat)
	{
		mmioClose(hmmio, 0);
		return 0;
	}

	/* Read the format chunk */
	if (mmioRead(hmmio, (HPSTR) lpWaveFormat, dwFmtSize) != (LONG) dwFmtSize)
	{
		GlobalFreePtr( lpWaveFormat );
		mmioClose(hmmio, 0);
		return 0;
	}
	
	/* Ascend out of the format subchunk */
	mmioAscend(hmmio, &mmckinfoSubchunk, 0);
	
	/* Find the data subchunk */
	mmckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	if (mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK))
	{
		GlobalFreePtr( lpWaveFormat );
		mmioClose(hmmio, 0);
		return 0;
	}

	/* Get the size of the data subchunk */
	dwDataSize = mmckinfoSubchunk.cksize;
	if (dwDataSize == 0L)
	{
		GlobalFreePtr( lpWaveFormat );
		mmioClose(hmmio, 0);
		return 0;
	}
		
	/* Allocate and lock memory for the waveform data. */
	lpWaveData = (HPSTR) GlobalAllocPtr(GMEM_MOVEABLE | GMEM_SHARE, dwDataSize );
	if (!lpWaveData)
	{
		GlobalFreePtr( lpWaveFormat );
		mmioClose(hmmio, 0);
		return 0;
	}

	/* Read the waveform data subchunk */
	if(mmioRead(hmmio, (HPSTR) lpWaveData, dwDataSize) != (LONG) dwDataSize)
	{
		GlobalFreePtr(lpWaveFormat);
		GlobalFreePtr(lpWaveData);
		mmioClose(hmmio, 0);
		return 0;
	}

	/* We're done with the file, close it */
	mmioClose(hmmio, 0);

	/* Allocate a waveform data header */
	lpWaveHdr = (WAVEHDR FAR*) GlobalAllocPtr(GMEM_MOVEABLE | GMEM_SHARE, (DWORD)sizeof(WAVEHDR));
	if (!lpWaveHdr)
	{
		GlobalFreePtr(lpWaveData);
		GlobalFreePtr(lpWaveFormat);
		return 0;
	}

	/* Set up WAVEHDR structure and prepare it to be written to wave device */
	lpWaveHdr->lpData = lpWaveData;
	lpWaveHdr->dwBufferLength = dwDataSize;
	lpWaveHdr->dwFlags = 0L;
	lpWaveHdr->dwLoops = 0L;
	lpWaveHdr->dwUser  = 0L;


	/* make sure wave device can play our format */
	if (waveOutOpen((LPHWAVEOUT)NULL,
					(WORD)dw_devid,
					(LPWAVEFORMAT)lpWaveFormat, 
					0L, 0L, WAVE_FORMAT_QUERY)) {
		GlobalFreePtr( lpWaveFormat );
        MessageBox (NULL, "Unsupported wave format.", "", MB_OK);
        return 0;
	}

	/* open the wave device corresponding to the line */
	if (waveOutOpen (	&hWave, 
						(WORD)dw_devid, 
						(LPWAVEFORMAT)lpWaveFormat, 
						(DWORD)hWnd, 
						0L, 
						CALLBACK_WINDOW))
	{
		GlobalFreePtr( lpWaveFormat );
        MessageBox (NULL, "Error opening wave device.", "", MB_OK);
        return 0;
	}

	GlobalFreePtr( lpWaveFormat );


	/* prepare the message header for playing */
	if (waveOutPrepareHeader (hWave, lpWaveHdr, sizeof(WAVEHDR))) {
       MessageBox (NULL, "Error preparing message header.", "", MB_OK);
        return 0;
	}

	/* play the message right from the data segment;  set the play message flag */
	if (waveOutWrite (hWave, lpWaveHdr, sizeof (WAVEHDR))) {
        MessageBox (NULL, "Error writing wave message.", "", MB_OK);
        return 0;
	}


	return hWave;

} /* end function (play message) */

