#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <tapi.h>
#include "playdemo.h"
#include "dial.h"

/* Global variables */
HINSTANCE	ghInstance;
HWND		ghWnd;

/* handles to wave devices */
HWAVEOUT h_wave=0;

char szAppName[] = "TapiDemo";

/*********************************************************/
int PASCAL WinMain (	HINSTANCE	hInstance,
						HINSTANCE	hPrevInstance,
						LPSTR		lpszCmdLine,
						int			nCmdShow	)
{
	WNDCLASS WndClass;
	MSG		 msg;

	ghInstance=hInstance;
	
	/* Register class */
	if (!hPrevInstance)
	{
		WndClass.lpszClassName	= szAppName;
		WndClass.hInstance		= hInstance;
		WndClass.lpfnWndProc	= (WNDPROC)MainWndProc;
		WndClass.hCursor		= LoadCursor(NULL,IDC_ARROW);
		WndClass.hIcon			= LoadIcon(NULL,IDI_APPLICATION);
		WndClass.lpszMenuName	= "MyMenu";
		WndClass.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
		WndClass.style			= CS_HREDRAW | CS_VREDRAW;
		WndClass.cbClsExtra		= 0;
		WndClass.cbWndExtra		= 0;

		RegisterClass(&WndClass);
	}

	/* Create main window */
	ghWnd = CreateWindow( szAppName, "Tapi Demonstration",
						 WS_OVERLAPPEDWINDOW,
						 CW_USEDEFAULT, 0,
						 CW_USEDEFAULT, 0,
						 NULL, NULL, hInstance, 0L );

	ShowWindow(ghWnd, nCmdShow);

	/* Enter message loop */
	while(GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	
	return msg.wParam;
}
	
/*********************************************************/
LRESULT __export FAR PASCAL MainWndProc(	HWND	hWnd,
											UINT	wMsgID,
											WPARAM	wParam,
											LPARAM	lParam )
{
	switch(wMsgID)
	{
		case WM_COMMAND:
			switch (wParam)
			{
				case IDM_DIAL:

					/* Make a phone call */
					if (!MakeACall())
					{
						perror("Can't make call");
					}
					return 0L;
			}
			break;

		case TAPI_REPLY:
			switch (lParam)
			{
				case TAPIERR_CONNECTED:

					/* Play the .wav file out to the phone */
					h_wave=playSound(WAVEDEVID,"test.wav",hWnd);
					if (!h_wave)
					{
						tapiRequestDrop(hWnd,0);
						perror("Unable to play sound");
					}
					return 0L;
					
				case TAPIERR_DROPPED:
					if (h_wave)
						waveOutReset (h_wave);
					return 0L;
			}

		case MM_WOM_DONE:

			/* done playing .wav file */

			/* Unprepare the header */
			waveOutUnprepareHeader((HWAVEOUT)wParam,(LPWAVEHDR)lParam, sizeof(WAVEHDR));
			/* close wave device */
			waveOutClose((HWAVEOUT)wParam);
			/* free the wave data */
			GlobalFreePtr (((LPWAVEHDR)lParam)->lpData);
			/* free the header */
			GlobalFreePtr ((LPWAVEHDR)lParam);

			/* clear h_wave to signify that wave output done */
			h_wave=0;

			/* hang up phone */
			tapiRequestDrop(hWnd, 0);

           return 0L;
 
		case WM_CLOSE:
			DestroyWindow(hWnd);
			return 0L;

		case WM_DESTROY:
			PostQuitMessage(0);
			return 0L;
	}
	return DefWindowProc(hWnd, wMsgID, wParam, lParam);
}

/*********************************************************/
/* Procedure to dial a phone call using TAPI */
int MakeACall()
{
	long tapiRetCode;

	/* Get name & number */
	if (DialogBox(ghInstance, "DialDialog", ghWnd, (DLGPROC)DialDialogProc) != IDOK)
		return 0;

	tapiRetCode = tapiRequestMediaCall(ghWnd,0,"wave",WAVEDEVIDSTR,0L,0L,
					szDialNumber,szAppName,szDialName,NULL);

	switch (tapiRetCode)
	{
		case 0:
			perror("Call placed successfully");
			break;
		case TAPIERR_NOREQUESTRECIPIENT:
			perror("No call manager app running");
			break;
		case TAPIERR_REQUESTQUEUEFULL:
			perror("Request queue full; try again later");
			break;
		case TAPIERR_INVALDESTADDRESS:
			perror("Invalid destination address");
			break;
		default:
			perror("Unrecognized error");
	}

	return (int)tapiRetCode;
}

void perror(char *Error)
{
	MessageBox(ghWnd, Error,	szAppName, MB_OK | MB_ICONEXCLAMATION);
}

