#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <tapi.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include "tam.h"
#include "async.h"

/* flags */
BOOL b_playingmessage;
BOOL b_playingbeep;
BOOL b_recording;

/* handles to wave devices */
HWAVE h_wave;
HWAVEIN h_wavein;

HANDLE hInst;                       //current instance
HANDLE h_wndmain;                   //handle to main window

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow);
BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FAR PASCAL MainWndProc(HWND hWnd, unsigned message, WORD wParam,
                            LONG lParam);

HLINE hLine=NULL;
DWORD dw_waveid=0;
HCALL h_call;                       /* handle to active call */

/* async ids */
LONG l_ansid;                       /* answer id */
LONG l_dropid;                      /* drop id */

/****************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
    PURPOSE: calls initialization function, processes message loop
****************************************************************************/
int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
MSG msg;

   lpCmdLine;

   if (!hPrevInstance)                  //Other instances of app running?
      if (!InitApplication(hInstance))  //Initialize shared things
          return (FALSE);               //Exits if unable to initialize
   //Perform initializations that apply to a specific instance
   if (!InitInstance(hInstance, nCmdShow))
      return (FALSE);
   //Acquire and dispatch messages until a WM_QUIT message is received.
   while (GetMessage(&msg,        //message structure
       NULL,                      //handle of window receiving the message
       NULL,                      //lowest message to examine
       NULL))                     //highest message to examine
    {
      TranslateMessage(&msg);     //Translates virtual key codes
      DispatchMessage(&msg);      //Dispatches message to window
    } //End while (not a quit message)
    return (msg.wParam);          //Returns the value from PostQuitMessage
} /* end function (WinMain) */
/****************************************************************************
    FUNCTION: InitApplication(HANDLE)
    PURPOSE: Initializes window data and registers window class
****************************************************************************/
BOOL InitApplication(HANDLE hInstance)
{
WNDCLASS  wc;

   //Fill in window class structure with parameters that describe the
   //main window.
   wc.style = NULL;                    //Class style(s).
   wc.lpfnWndProc = (WNDPROC)MainWndProc;  //Function to retrieve messages for
                                           //windows of this class.
   wc.cbClsExtra = 0;                  //No per-class extra data.
   wc.cbWndExtra = 0;                  //No per-window extra data.
   wc.hInstance = hInstance;           //Application that owns the class.
   wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = GetStockObject(WHITE_BRUSH);
   wc.lpszMenuName =  "tamMenu";       //Name of menu resource in .RC file.
   wc.lpszClassName = "TAMWClass";     //Name used in call to CreateWindow.

   //Register the window class and return success/failure code.
   return (RegisterClass(&wc));
} //End function (InitApplication)
/****************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)
    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/
BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{
HWND hWnd;

   //Save the instance handle in static variable, which will be used in
   //many subsequence calls from this application to Windows.
   hInst = hInstance;
   //Create a main window for this application instance.
   hWnd = h_wndmain = CreateWindow(
            "TAMWClass",                    // See RegisterClass() call.
            "Telephone Answering Machine",  // Text for window title bar.
            WS_OVERLAPPEDWINDOW,            // Window style.
            CW_USEDEFAULT,                  // Default horizontal position.
            CW_USEDEFAULT,                  // Default vertical position.
            CW_USEDEFAULT,                  // Default width.
            CW_USEDEFAULT,                  // Default height.
            NULL,                           // Overlapped windows have no parent
            NULL,                           // Use the window class menu.
            hInstance,                      // This instance owns this window.
            NULL                            // Pointer not needed.
             );
   //If window could not be created, return "failure"
   if (!hWnd)
   return (FALSE);
   //Make the window visible; update its client area; and return "success"
   ShowWindow(hWnd, nCmdShow);              //Show the window
   UpdateWindow(hWnd);                      //Sends WM_PAINT message

   return (TRUE);               //Returns the value from PostQuitMessage
} //End function (InitInstance)
/****************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)
    PURPOSE:  Processes messages
****************************************************************************/
long FAR PASCAL MainWndProc(HWND hWnd, unsigned message, WORD wParam,
                            LONG lParam)
{
static HLINEAPP hApp=0;
static DWORD dwVerAPI;
static DWORD dwLineID;
HDC hDC;
PAINTSTRUCT ps;
LONG lrc;

    switch (message)
    {
      /* the window msg function is notified when the buffer has played out */
      case MM_WOM_DONE:
         /* Unprepare the header */
         waveOutUnprepareHeader((HWAVEOUT)wParam,(LPWAVEHDR)lParam,
                                 sizeof(WAVEHDR));
         /* close wave device */
         waveOutClose((HWAVEOUT)wParam);
         /* free the wave data */
         GlobalFreePtr (((LPWAVEHDR)lParam)->lpData);
         /* free the header */
         GlobalFreePtr ((LPWAVEHDR)lParam);
         if (b_playingmessage) {
            b_playingmessage = FALSE;
            /* play the beep */
            if (h_wave=playSound (dw_waveid,"beep.wav", h_wndmain))
               b_playingbeep = TRUE;
            break;
         }
         if (b_playingbeep) {
            b_playingbeep = FALSE;
            /* start recording message */
            if (h_wavein=recordMessage (dw_waveid))
               b_recording = TRUE;
            break;
         }
         break;

      case MM_WIM_DATA:
         waveInUnprepareHeader (h_wavein, (LPWAVEHDR)lParam, sizeof(WAVEHDR));
         /* close wave device */
         waveInClose (h_wavein);
         /* drop the call */
         if (h_call) {
            if ((l_dropid = lineDrop (h_call, NULL, 0)) < 0) {
               lineError (l_dropid);
               MessageBox (h_wndmain, "Error dropping call", "", MB_OK);
            }
            WaitForAsyncReturn (&l_dropid);
         }
         /* save the message */
         saveMessage ();
         /* free 60s message buffer */
         hfree (((LPWAVEHDR)lParam)->lpData);
         b_recording = FALSE;
         break;

      case WM_PAINT:
         hDC=BeginPaint(hWnd,&ps);
         EndPaint (hWnd, &ps);
         break;

      case WM_COMMAND:
         switch (wParam) {
            case IDM_AUTOANSWER:
               if ((lrc = lineOpen(hApp, dwLineID, &hLine, dwVerAPI,
                    0 /* no extensions */,
                    NULL /* instance data */, LINECALLPRIVILEGE_OWNER,
                    LINEMEDIAMODE_UNKNOWN | LINEMEDIAMODE_INTERACTIVEVOICE,
                    NULL)) < 0) {
                  lineError (lrc);
                  PostQuitMessage (0);
                  break;
               }

               /* get the id of the wave audio device assoc with the line */
               if ((dw_waveid = getWaveDeviceID(hLine)) < 0) {
                  break;
               }

               break;
         default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
         } //End switch (wParam)
         break;

      case WM_CREATE:
         /* initialize line device, neg. versions, etc...    */
         if ((doInitStuff (&hApp, &dwVerAPI, &dwLineID) < 0)
                           || ((dwVerAPI || dwLineID)==0)) {
            MessageBox (hWnd, "Error initializing line", "", MB_OK);
            PostQuitMessage (0);
         }
         break;

      case WM_DESTROY:
         if (h_call) {
            /* drop the call */
            if ((l_dropid = lineDrop (h_call, NULL, 0)) < 0) {
               lineError (l_dropid);
               MessageBox (h_wndmain, "Error dropping call", "", MB_OK);
            }
            WaitForAsyncReturn (&l_dropid);
         }
         if (hLine)
            lineClose (hLine);
         if (hApp)
            lineShutdown (hApp);
          PostQuitMessage(0);
          break;
      case WM_TIMER:
          break;
      default:
          return (DefWindowProc(hWnd, message, wParam, lParam));
       } //End switch (message)

   return (NULL);
} //End function (MainWndProc)
/****************************************************************************
    FUNCTION: lineCallback
    PURPOSE:  callback function handles line events
****************************************************************************/
VOID FAR PASCAL __export lineCallback (DWORD dwDevice, DWORD dwMsg,
                                       DWORD dwCallbackInst,
                                       DWORD dwParam1, DWORD dwParam2,
                                       DWORD dwParam3)
{
   dwCallbackInst;

   switch (dwMsg)
   {
      case LINE_MONITORMEDIA:
         /* if it's a fax call, hand it off to fax application */
         if (dwParam1 & LINEMEDIAMODE_G3FAX) {
            /* classify the call */
            lineSetMediaMode (h_call, LINEMEDIAMODE_G3FAX);
            /* hand it off to fax application */
            if (lineHandoff (h_call, NULL, LINEMEDIAMODE_G3FAX) < 0) {
               /* could not hand off the fax - hang up in frustration */
               /* drop the call */
               if ((l_dropid = lineDrop (h_call, NULL, 0)) < 0) {
                  lineError (l_dropid);
                  MessageBox (h_wndmain, "Error dropping call", "", MB_OK);
               }
               WaitForAsyncReturn (&l_dropid);
            }
            /* clean up after call */
            /* since we don't drop the call, we must explicitly deallocate it
               instead of relying on LINE_REPLY to do it */
            if (h_call)
               lineDeallocateCall (h_call);
            doneCall ();
         }
         break;

      case LINE_REPLY:
         /* call drop completion */
         if ((LONG)dwParam1 == l_dropid) {
            l_dropid = 0;
            if (h_call)
               lineDeallocateCall (h_call);
            h_call = NULL;
         }
         /* call answer completion */
         if ((LONG)dwParam1 == l_ansid)
            l_ansid = 0;
         /* error */
         if ((LONG)dwParam2  < 0)
            lineError ((LONG)dwParam2);
         AsyncReturn (dwParam1, dwParam2);
         break;

      case LINE_CALLSTATE:
         switch (dwParam1)
         {
            case LINECALLSTATE_DIALTONE:
            case LINECALLSTATE_IDLE:
               /* clean up after call */
               doneCall ();
               break;
            case LINECALLSTATE_CONNECTED:
               if (h_wave=playSound (dw_waveid,"msg.wav", h_wndmain))
                  b_playingmessage = TRUE;
               break;
            case LINECALLSTATE_OFFERING:
               /* The call is being offered, signaling the arrival
               of a  new call.  check if we are the owner of the call,
               and if we are, then answer it. */
               if (dwParam3 == LINECALLPRIVILEGE_OWNER) {
                  if ((l_ansid=lineAnswer (h_call = (HCALL)dwDevice, NULL, 0))
                     < 0) {
                     lineError (l_ansid);
                     MessageBox (h_wndmain, "Error answering call", "", MB_OK);
                     return;
                  }
                  WaitForAsyncReturn (&l_ansid);
                  /* set notification of fax detection for later handoff */
                  lineMonitorMedia (h_call, LINEMEDIAMODE_G3FAX);
               } /* end if (we own this call) */
               break;
      } /* end switch (on call state) */
   } /* end switch (on message) */
} /* end function (lineCallback) */
/****************************************************************************
    FUNCTION: doInitStuff
    PURPOSE:  perform initializaions
****************************************************************************/
#define APIHIVERSION    0x00010001              /* 1.01 */
#define APILOWVERSION   0x00010001              /* 1.01 */

int doInitStuff (HLINEAPP *ph_app, DWORD *pdw_ver, DWORD *pdw_lineid)
{
LINEEXTENSIONID ext_id;
DWORD dw_numdevs, i;
LONG lrc;
LINEDEVCAPS *p_lineDevCaps;

   *pdw_lineid = *pdw_ver = 0;
   p_lineDevCaps = (LINEDEVCAPS *) calloc (1, sizeof(LINEDEVCAPS)+1000);

   /* initialize application's use of the telephone API */
   if (lineInitialize(ph_app, hInst, lineCallback, NULL, /* 0, */
                      &dw_numdevs) < 0) {
      /* return error */
      MessageBox (h_wndmain, "Error initializing line", "", MB_OK);
      return -1;
   }

   /* negotiate the line API version */
   if ((lrc = lineNegotiateAPIVersion(*ph_app,
                                      0,
                                      APILOWVERSION, APIHIVERSION, pdw_ver,
                                      &ext_id)) < 0)      {
      lineError (lrc);
      lineShutdown (*ph_app);
      *ph_app = NULL;
      MessageBox (h_wndmain, "Error negotiating version", "", MB_OK);
      return -1;
   }

   /* for each line device, get capabilities until we find one that supports
   interactive voice */
   p_lineDevCaps->dwTotalSize = sizeof(LINEDEVCAPS) + 1000;
   for (i=0; i < dw_numdevs; i++) {
      lineGetDevCaps(*ph_app, i, *pdw_ver, 0, p_lineDevCaps);
      if (p_lineDevCaps->dwMediaModes & LINEMEDIAMODE_INTERACTIVEVOICE) {
         *pdw_lineid = i;
         break;
      }
   }

   free (p_lineDevCaps);

   return 0;
} /* end function (doInitStuff) */
/****************************************************************************
    FUNCTION: lineError
    PURPOSE:  line error messages
****************************************************************************/
void lineError (LONG lrc)
{
      switch (lrc) {
         case LINEERR_INVALAPPHANDLE:
            MessageBox (h_wndmain, "Invalid app handle.", "", MB_OK);
            break;
         case LINEERR_BADDEVICEID:
            MessageBox (h_wndmain,
                    "The specified line device ID is out of range.", "", MB_OK);
            break;
         case LINEERR_INCOMPATIBLEAPIVERSION:
            MessageBox (h_wndmain, "Incompatible API version.", "", MB_OK);
            break;
         case LINEERR_INCOMPATIBLEEXTVERSION:
            MessageBox (h_wndmain, "Incompatible extension version.","", MB_OK);
            break;
         case LINEERR_NOMEM:
            MessageBox (h_wndmain, "No memory",
                        "", MB_OK);
            break;
         case LINEERR_NODRIVER:
            MessageBox (h_wndmain, "No driver loaded",
                        "", MB_OK);
            break;
         case LINEERR_RESOURCEUNAVAIL:
            MessageBox (h_wndmain, "Resource overcommittment",
                        "", MB_OK);
            break;
         case LINEERR_INVALPRIVSELECT:
            MessageBox (h_wndmain, "Requested invalid priviledges",
                        "", MB_OK);
            break;
         case LINEERR_INVALMEDIAMODE:
            MessageBox (h_wndmain, "Requested invalid media mode",
                        "", MB_OK);
            break;
         case LINEERR_LINEMAPPERFAILED:
            MessageBox (h_wndmain, "Line mapper failed",
                        "", MB_OK);
            break;
         case LINEERR_INVALPOINTER:
            MessageBox (h_wndmain, "The specified pointer parameter is invalid.",
                        "", MB_OK);
            break;
         case LINEERR_OPERATIONFAILED:
            MessageBox (h_wndmain, "Operation failed.", "", MB_OK);
            break;
         case LINEERR_INVALLINEHANDLE:
            MessageBox (h_wndmain, "Invalid line handle",
                        "", MB_OK);
            break;
         case LINEERR_INVALADDRESSID:
            MessageBox (h_wndmain, "Invalid address id",
                        "", MB_OK);
            break;
         case LINEERR_INVALCALLHANDLE:
            MessageBox (h_wndmain, "Invalid call handle",
                        "", MB_OK);
            break;
         case LINEERR_INVALCALLSELECT:
            MessageBox (h_wndmain, "Invalid call selection",
                        "", MB_OK);
            break;
         case LINEERR_NODEVICE:
            MessageBox (h_wndmain, "No associated device for given class",
                        "", MB_OK);
            break;
      }
}
/****************************************************************************
   FUNCTION: getWaveDeviceID
   PURPOSE : get the id of the wave device corresponding to the line
   COMMENTS: wave devices use an unsigned integer id.  we must allocate enough
             space for this id to be returned from the driver.
****************************************************************************/
LONG getWaveDeviceID (HLINE h_line)
{
LONG lrc;
WORD w_id;
/* allocate 2 extra bytes for wave id. */
VARSTRING *p_varstr = (VARSTRING *)calloc (sizeof(VARSTRING) + 2, 1);

   p_varstr->dwTotalSize =  sizeof (VARSTRING) + 2;
   if ((lrc = lineGetID (h_line, 0 /* not an address */, 0 /* not a call */,
                         LINECALLSELECT_LINE /* it's a line! */, p_varstr,
                         "wave" /* wave device */)) < 0) {
      lineError (lrc);
      PostQuitMessage (0);
      return -1;
   }
   /* if the returned id was not binary, something is wrong */
   if (p_varstr->dwStringFormat != STRINGFORMAT_BINARY) {
      MessageBox (h_wndmain, "Bad wave device id",
                  "", MB_OK);
      PostQuitMessage (0);
      return -1;
   }
   /* get id */
   w_id = (WORD) *((BYTE *)p_varstr+sizeof(VARSTRING));
   free (p_varstr);

   return (LONG)w_id;
} /* end function (getWaveDeviceID) */
/****************************************************************************
    FUNCTION: doneCall
    PURPOSE:  end-of-call processing
****************************************************************************/
void doneCall ()
{
   /* if playing or recording a message, close up */
   if (b_playingmessage || b_playingbeep) {
      b_playingmessage = b_playingbeep = FALSE;
      /* reset wave device */
      waveOutReset (h_wave);
      /* drop the call */
      if ((l_dropid = lineDrop (h_call, NULL, 0)) < 0) {
         lineError (l_dropid);
         MessageBox (h_wndmain, "Error dropping call", "", MB_OK);
         return;
      }
      WaitForAsyncReturn (&l_dropid);
   }
   /* close up if recording */
   else if (b_recording) {
      b_recording = FALSE;
      waveInStop (h_wavein);
   }

} /* end function (doneCall) */
